package io.reisub.dreambot.util.event;

import java.util.EventListener;
import java.util.LinkedList;
import java.util.List;

public abstract class AbstractEvent {
    private Thread thread;
    protected volatile boolean run = false;
    protected List<EventListener> listeners = new LinkedList<>();

    public void start() {
        if (run) return;

        run = true;
        thread = new Thread(this::run);
        thread.start();
    }

    public void stop() {
        run = false;
        thread = null;
    }

    public abstract void run();

    public void addListener(EventListener listener) {
        this.listeners.add(listener);

        if (this.listeners.size() == 1) this.start();
    }

    public void removeListener(EventListener listener) {
        this.listeners.remove(listener);

        if (this.listeners.size() == 0) this.stop();
    }

    public void removeAllListeners() {
        this.listeners.clear();

        this.stop();
    }
}
